#include <delays.h>
#include "mylcd.h"
#include <xlcd.h>
#include <stdlib.h>
#include <string.h>

void initLCD( void ) {
	// configure external LCD
	DelayPORXLCD();
	OpenXLCD( FOUR_BIT & LINES_5X7, 0 );
	delayMicro(160);
	OpenXLCD( FOUR_BIT & LINES_5X7, 1 );
	delayMicro(160);
	WriteCmdXLCD(CURSOR_OFF);
}

//18 instruction delay
void DelayFor18TCY( void )
{
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
	Nop();
}

//15-20ms delay
void DelayPORXLCD( void )
{
	Delay1KTCYx(20 * DELAY_PER_MS); //Delay of 15ms
	return;
}

//8ms delay
void DelayXLCD( void )
{
	Delay1KTCYx(8 * DELAY_PER_MS); //Delay of 8ms
	return;
}

//Put a character on the lcd
void putchar( char c ) {
	delayMicro(160);
	putcXLCD(c);
}

//accepts 0 to 3, sets cursor to that line
void setLine(int i) {

	int LINE[2];
	
	LINE[0] =0x00;
	LINE[1] =0x40;

	CS_PIN = (i % 4)/2;	//select chip 2 if line = 2 or 3
	WriteCmdXLCD(0x80 | LINE[i%2]);
	return;
}

//Set absolute position of cursor
void setPos(int row, int col) {

	int LINE[2];
	int ddr_addr;

	LINE[0] =0x00;
	LINE[1] =0x40;

	CS_PIN = (row % 4)/2;	//select chip 2 if row = 2 or 3
	ddr_addr = LINE[row%2] + col;
	delayMicro(160);
	WriteCmdXLCD(0x80 | ddr_addr);
	return;
}

//Clear a line on the lcd
void clearLine( int lineNum ) {
	setLine(lineNum);
	putrsXLCD("                                        "); 
	setLine(lineNum);
	return;
}


void delayMicro( unsigned int microsecs ) {
	unsigned int delayCount = (int)((double)microsecs * DELAY_PER_MS / ((double)10));
	//Delay functions take char as input
	while (delayCount > 255) {
		Delay10TCYx(256);
		delayCount -= 256;
	}
	Delay10TCYx(delayCount);
	return;
}

void delay (unsigned int millisecs) {
	unsigned int delayCount = (int)((double)millisecs * DELAY_PER_MS / ((double)10));
	//Delay functions take char as input
	while (delayCount > 255) {
		Delay10KTCYx(256);
		delayCount -= 256;
	}
	Delay10KTCYx(delayCount);
	return;
}


//expensive, hacked version -- may want to eventually optimize this
void writeFloatToLCD ( float f ) {
	int fint = (int)f;
	int fint2;
	char str[10];
	char str2[4];
	char decimalpoint[2] = ".";
	char zeroes2[3] = "00";
	char zeroes1[2] = "0";
	
	//output >1 part
	itoa(fint, str);
	//while(BusyXLCD());
	//putsXLCD(str);
	
	//output decimal point
	while(BusyXLCD());
	//strcat(str, ".");
	strcat(str, decimalpoint);
	//putrsXLCD(".");

	//output 3 decimal places
	fint2 = (int)(f * 1000);
	fint2 -= fint*1000;
	if (f < 10) {
		strcat(str,zeroes2);
	} else if (f < 100) {
		strcat(str,zeroes1);
	}

	itoa(fint2, str2);
	strcat(str, str2);
	
	while(BusyXLCD());
	putsXLCD(str);
}
	
	

/*
void waitAndWrite(int i, const rom char *buffer) {
		while( BusyXLCD() );	//wait until LCD is not busy
		setLine(i);
		putrsXLCD(buffer);
}
*/
